function [drift_x, drift_y] = driftcorrection(bright_field_images, pixel_size, upsample_scale, output_file)
%
%[drift_x, drift_y] = driftcorrection[bright_field_images,pixel_size,output_file]
%
%compute drifts between the first frame bright field image and other frames
%bright field image;
%bright_field_images: bright field images with tiff format.
%pixel_size: the pixel size of bright field images with nanometer.
%output_file: the file name for save the drift result, as frames, drift_x,
%drift_y
% By Yunqing Tang, 03/07/2014

InfoImage=imfinfo(bright_field_images);
width=InfoImage(1).Width;
height=InfoImage(1).Height;
all_frames=length(InfoImage);

fid_fitting=fopen(output_file,'w');
drift_x = zeros(all_frames,1);
drift_y = zeros(all_frames,1);

[ref_image, map] = imread(bright_field_images,1);
number_of_data = height * width;
ref_image = double(ref_image);
ref_image = gaussianSmooth(ref_image);
ref_list = reshape(ref_image, number_of_data, 1);
centerx =  height/2;
centery = width/2;

for frame = 1:all_frames
    [uncorrcted_image,map] = imread(bright_field_images,frame);
    uncorrcted_image = double(uncorrcted_image);
    uncorrcted_image = gaussianSmooth(uncorrcted_image);
    
    
    %%estimate initial value of parameter use FFT methods
    scale_ref_image = imresize(ref_image, upsample_scale); %upsampled reference image
    scale_uncorrcted_image = imresize(uncorrcted_image, upsample_scale); %upsampled uncorrcted image
    %compute cross correltion image between upsampled reference image and upsampled uncorrcted image
    CrossCorrelationImage = computeUpsampledCrossCorrelation(scale_ref_image,scale_uncorrcted_image);
    %get the position of maximum
    [rows,cols]=find(CrossCorrelationImage==max(max(CrossCorrelationImage)));
    dx_initial = rows/upsample_scale - centerx;
    dy_initial = cols/upsample_scale - centery;
    %get the initial values of parameters
    initial_parameters =[1; dx_initial; dy_initial]';
    
    %nonlinear fitting
    parameters = nlinfit(uncorrcted_image,ref_list,@model_function,initial_parameters);
    %change pixels to nm
    drift_x(frame) = -parameters(2)*pixel_size;
    drift_y(frame) = -parameters(3)*pixel_size;
    disp(frame-1);
end

for frame = 1:all_frames
    fprintf(fid_fitting, '%d %f %f\n', frame-1, drift_x(frame), drift_y(frame));
end

fclose(fid_fitting);






